@@ -90,8 +90,6 @@
 
 #include "core/logging.h"
 #include "core/messagehandler.h"
-
-#include "fmpsparser.h"
 #include "core/timeconstants.h"
 
 class FileRefFactory {
@@ -300,15 +298,6 @@ void TagReader::ReadFile(const QString &filename, pb::tagreader::SongMetadata *s
         }
       }
 
-      // Parse FMPS frames
-      for (uint i = 0; i < map["TXXX"].size(); ++i) {
-        const TagLib::ID3v2::UserTextIdentificationFrame *frame = dynamic_cast<const TagLib::ID3v2::UserTextIdentificationFrame*>(map["TXXX"][i]);
-
-        if (frame && frame->description().startsWith("FMPS_")) {
-          ParseFMPSFrame(TStringToQString(frame->description()), TStringToQString(frame->fieldList()[1]), song);
-        }
-      }
-
     }
   }
 
@@ -402,7 +391,7 @@ void TagReader::ReadFile(const QString &filename, pb::tagreader::SongMetadata *s
 
   if (compilation.isEmpty()) {
     // well, it wasn't set, but if the artist is VA assume it's a compilation
-    if (QStringFromStdString(song->artist()).toLower() == "various artists") {
+    if (QStringFromStdString(song->artist()).toLower() == "various artists" || QStringFromStdString(song->albumartist()).toLower() == "various artists") {
       song->set_compilation(true);
     }
   }
@@ -523,33 +512,6 @@ void TagReader::ParseAPETag(const TagLib::APE::ItemListMap &map, const QTextCode
 
 }
 
-void TagReader::ParseFMPSFrame(const QString &name, const QString &value, pb::tagreader::SongMetadata *song) const {
-
-  qLog(Debug) << "Parsing FMPSFrame" << name << ", " << value;
-  FMPSParser parser;
-
-  if (!parser.Parse(value) || parser.is_empty()) return;
-
-  QVariant var;
-
-  if (name == "FMPS_PlayCount") {
-    var = parser.result()[0][0];
-    if (var.type() == QVariant::Double) {
-      song->set_playcount(var.toDouble());
-    }
-  }
-  else if (name == "FMPS_PlayCount_User") {
-    // Take a user playcount only if there's no playcount already set
-    if (song->playcount() == 0 && parser.result()[0].count() >= 2) {
-      var = parser.result()[0][1];
-      if (var.type() == QVariant::Double) {
-        song->set_playcount(var.toDouble());
-      }
-    }
-  }
-
-}
-
 void TagReader::SetVorbisComments(TagLib::Ogg::XiphComment *vorbis_comments, const pb::tagreader::SongMetadata &song) const {
 
   vorbis_comments->addField("COMPOSER", StdStringToTaglibString(song.composer()), true);
@@ -570,7 +532,8 @@ void TagReader::SetVorbisComments(TagLib::Ogg::XiphComment *vorbis_comments, con
 
 bool TagReader::SaveFile(const QString &filename, const pb::tagreader::SongMetadata &song) const {
 
-  if (filename.isNull() || filename.isEmpty()) return false;
+  if (filename.isEmpty()) return false;
+
   qLog(Debug) << "Saving tags to" << filename;
   std::unique_ptr<TagLib::FileRef> fileref(factory_->GetFileRef(filename));;
   if (!fileref || fileref->isNull()) return false;
