@@ -30,11 +30,11 @@ class QIODevice;
 
 class TagReaderWorker : public AbstractMessageHandler<pb::tagreader::Message> {
 public:
-  TagReaderWorker(QIODevice *socket, QObject *parent = nullptr);
+  explicit TagReaderWorker(QIODevice *socket, QObject *parent = nullptr);
 
 protected:
-  void MessageArrived(const pb::tagreader::Message &message);
-  void DeviceClosed();
+  void MessageArrived(const pb::tagreader::Message &message) override;
+  void DeviceClosed() override;
 
 private:
   TagReader tag_reader_;
