@@ -77,6 +77,8 @@
 #include "albumcoverexporter.h"
 #include "albumcoverfetcher.h"
 #include "albumcoverloader.h"
+#include "albumcoverloaderoptions.h"
+#include "albumcoverloaderresult.h"
 #include "albumcovermanagerlist.h"
 #include "coversearchstatistics.h"
 #include "coversearchstatisticsdialog.h"
@@ -194,13 +196,13 @@ void AlbumCoverManager::Init() {
   ui_->albums->installEventFilter(this);
 
   // Connections
-  connect(ui_->artists, SIGNAL(currentItemChanged(QListWidgetItem*,QListWidgetItem*)), SLOT(ArtistChanged(QListWidgetItem*)));
+  connect(ui_->artists, SIGNAL(currentItemChanged(QListWidgetItem*, QListWidgetItem*)), SLOT(ArtistChanged(QListWidgetItem*)));
   connect(ui_->filter, SIGNAL(textChanged(QString)), SLOT(UpdateFilter()));
   connect(filter_group, SIGNAL(triggered(QAction*)), SLOT(UpdateFilter()));
   connect(ui_->view, SIGNAL(clicked()), ui_->view, SLOT(showMenu()));
   connect(ui_->button_fetch, SIGNAL(clicked()), SLOT(FetchAlbumCovers()));
   connect(ui_->export_covers, SIGNAL(clicked()), SLOT(ExportCovers()));
-  connect(cover_fetcher_, SIGNAL(AlbumCoverFetched(const quint64, const QUrl&, const QImage&, const CoverSearchStatistics&)), SLOT(AlbumCoverFetched(const quint64, const QUrl&, const QImage&, const CoverSearchStatistics&)));
+  connect(cover_fetcher_, SIGNAL(AlbumCoverFetched(quint64, QUrl, QImage, CoverSearchStatistics)), SLOT(AlbumCoverFetched(quint64, QUrl, QImage, CoverSearchStatistics)));
   connect(ui_->action_fetch, SIGNAL(triggered()), SLOT(FetchSingleCover()));
   connect(ui_->albums, SIGNAL(doubleClicked(QModelIndex)), SLOT(AlbumDoubleClicked(QModelIndex)));
   connect(ui_->action_add_to_playlist, SIGNAL(triggered()), SLOT(AddSelectedToPlaylist()));
@@ -216,7 +218,7 @@ void AlbumCoverManager::Init() {
     ui_->splitter->setSizes(QList<int>() << 200 << width() - 200);
   }
 
-  connect(app_->album_cover_loader(), SIGNAL(ImageLoaded(const quint64, const QUrl&, const QImage&)), SLOT(CoverImageLoaded(const quint64, const QUrl&, const QImage&)));
+  connect(app_->album_cover_loader(), SIGNAL(AlbumCoverLoaded(quint64, AlbumCoverLoaderResult)), SLOT(AlbumCoverLoaded(quint64, AlbumCoverLoaderResult)));
 
   cover_searcher_->Init(cover_fetcher_);
 
@@ -248,6 +250,9 @@ void AlbumCoverManager::closeEvent(QCloseEvent *e) {
   // Cancel any outstanding requests
   CancelRequests();
 
+  ui_->artists->clear();
+  ui_->albums->clear();
+
 }
 
 void AlbumCoverManager::LoadGeometry() {
@@ -392,7 +397,7 @@ void AlbumCoverManager::ArtistChanged(QListWidgetItem *current) {
     }
 
     if (!info.art_automatic.isEmpty() || !info.art_manual.isEmpty()) {
-      quint64 id = app_->album_cover_loader()->LoadImageAsync(cover_loader_options_, info.art_automatic, info.art_manual, info.first_url.toLocalFile());
+      quint64 id = app_->album_cover_loader()->LoadImageAsync(cover_loader_options_, info.art_automatic, info.art_manual, info.first_url);
       item->setData(Role_PathAutomatic, info.art_automatic);
       item->setData(Role_PathManual, info.art_manual);
       cover_loading_tasks_[id] = item;
@@ -403,17 +408,15 @@ void AlbumCoverManager::ArtistChanged(QListWidgetItem *current) {
 
 }
 
-void AlbumCoverManager::CoverImageLoaded(const quint64 id, const QUrl &cover_url, const QImage &image) {
-
-  Q_UNUSED(cover_url);
+void AlbumCoverManager::AlbumCoverLoaded(const quint64 id, const AlbumCoverLoaderResult &result) {
 
   if (!cover_loading_tasks_.contains(id)) return;
 
   QListWidgetItem *item = cover_loading_tasks_.take(id);
 
-  if (image.isNull()) return;
+  if (result.image_scaled.isNull()) return;
 
-  item->setIcon(QPixmap::fromImage(image));
+  item->setIcon(QPixmap::fromImage(result.image_scaled));
   UpdateFilter();
 
 }
@@ -488,7 +491,7 @@ void AlbumCoverManager::FetchAlbumCovers() {
     if (item->isHidden()) continue;
     if (ItemHasCover(*item)) continue;
 
-    quint64 id = cover_fetcher_->FetchAlbumCover(EffectiveAlbumArtistName(*item), item->data(Role_AlbumName).toString(), true);
+    quint64 id = cover_fetcher_->FetchAlbumCover(EffectiveAlbumArtistName(*item), item->data(Role_AlbumName).toString(), QString(), true);
     cover_fetching_tasks_[id] = item;
     jobs_++;
   }
@@ -623,7 +626,7 @@ void AlbumCoverManager::ShowCover() {
 void AlbumCoverManager::FetchSingleCover() {
 
   for (QListWidgetItem *item : context_menu_items_) {
-    quint64 id = cover_fetcher_->FetchAlbumCover(EffectiveAlbumArtistName(*item), item->data(Role_AlbumName).toString(), false);
+    quint64 id = cover_fetcher_->FetchAlbumCover(EffectiveAlbumArtistName(*item), item->data(Role_AlbumName).toString(), QString(), false);
     cover_fetching_tasks_[id] = item;
     jobs_++;
   }
@@ -670,13 +673,13 @@ void AlbumCoverManager::SaveCoverToFile() {
     image = no_cover_image_;
   }
   else {
-    if (!song.art_manual().isEmpty() && !song.art_manual().isLocalFile() && QFile::exists(song.art_manual().toLocalFile())) {
+    if (!song.art_manual().isEmpty() && song.art_manual().isLocalFile() && QFile::exists(song.art_manual().toLocalFile())) {
       image = QImage(song.art_manual().toLocalFile());
     }
     else if (!song.art_manual().isEmpty() && !song.art_manual().path().isEmpty() && song.art_manual().scheme().isEmpty() && QFile::exists(song.art_manual().path())) {
       image = QImage(song.art_manual().path());
     }
-    else if (!song.art_automatic().isEmpty() && !song.art_automatic().isLocalFile() && QFile::exists(song.art_automatic().toLocalFile())) {
+    else if (!song.art_automatic().isEmpty() && song.art_automatic().isLocalFile() && QFile::exists(song.art_automatic().toLocalFile())) {
       image = QImage(song.art_automatic().toLocalFile());
     }
     else if (!song.art_automatic().isEmpty() && !song.art_automatic().path().isEmpty() && song.art_automatic().scheme().isEmpty() && QFile::exists(song.art_automatic().path())) {
@@ -799,19 +802,19 @@ SongMimeData *AlbumCoverManager::GetMimeDataForAlbums(const QModelIndexList &ind
   SongList songs = GetSongsInAlbums(indexes);
   if (songs.isEmpty()) return nullptr;
 
-  SongMimeData *data = new SongMimeData;
-  data->backend = collection_backend_;
-  data->songs = songs;
-  return data;
+  SongMimeData *mimedata = new SongMimeData;
+  mimedata->backend = collection_backend_;
+  mimedata->songs = songs;
+  return mimedata;
 
 }
 
 void AlbumCoverManager::AlbumDoubleClicked(const QModelIndex &index) {
 
-  SongMimeData *data = GetMimeDataForAlbums(QModelIndexList() << index);
-  if (data) {
-    data->from_doubleclick_ = true;
-    emit AddToPlaylist(data);
+  SongMimeData *mimedata = GetMimeDataForAlbums(QModelIndexList() << index);
+  if (mimedata) {
+    mimedata->from_doubleclick_ = true;
+    emit AddToPlaylist(mimedata);
   }
 
 }
@@ -822,10 +825,10 @@ void AlbumCoverManager::AddSelectedToPlaylist() {
 
 void AlbumCoverManager::LoadSelectedToPlaylist() {
 
-  SongMimeData *data = GetMimeDataForAlbums(ui_->albums->selectionModel()->selectedIndexes());
-  if (data) {
-    data->clear_first_ = true;
-    emit AddToPlaylist(data);
+  SongMimeData *mimedata = GetMimeDataForAlbums(ui_->albums->selectionModel()->selectedIndexes());
+  if (mimedata) {
+    mimedata->clear_first_ = true;
+    emit AddToPlaylist(mimedata);
   }
 
 }
@@ -889,7 +892,7 @@ void AlbumCoverManager::ExportCovers() {
 
 }
 
-void AlbumCoverManager::UpdateExportStatus(int exported, int skipped, int max) {
+void AlbumCoverManager::UpdateExportStatus(const int exported, const int skipped, const int max) {
 
   progress_bar_->setValue(exported);
 
@@ -943,4 +946,3 @@ QImage AlbumCoverManager::GenerateNoCoverImage(const QIcon &no_cover_icon) const
 bool AlbumCoverManager::ItemHasCover(const QListWidgetItem &item) const {
   return item.icon().cacheKey() != no_cover_item_icon_.cacheKey();
 }
-
