@@ -2,6 +2,7 @@
  * Strawberry Music Player
  * This code was part of Clementine (GlobalSearch)
  * Copyright 2012, David Sansome <me@davidsansome.com>
+ * Copyright 2018, Jonas Kvinge <jonas@jkvinge.net>
  *
  * Strawberry is free software: you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,12 +33,14 @@
 
 #include "core/mimedata.h"
 #include "core/iconloader.h"
-#include "internetsearch.h"
+#include "internetsongmimedata.h"
+#include "internetservice.h"
 #include "internetsearchmodel.h"
+#include "internetsearchview.h"
 
-InternetSearchModel::InternetSearchModel(InternetSearch *engine, QObject *parent)
+InternetSearchModel::InternetSearchModel(InternetService *service, QObject *parent)
     : QStandardItemModel(parent),
-      engine_(engine),
+      service_(service),
       proxy_(nullptr),
       use_pretty_covers_(true),
       artist_icon_(IconLoader::Load("folder-sound")),
@@ -45,30 +48,26 @@ InternetSearchModel::InternetSearchModel(InternetSearch *engine, QObject *parent
       {
 
   group_by_[0] = CollectionModel::GroupBy_AlbumArtist;
-  group_by_[1] = CollectionModel::GroupBy_Album;
+  group_by_[1] = CollectionModel::GroupBy_AlbumDisc;
   group_by_[2] = CollectionModel::GroupBy_None;
 
   no_cover_icon_ = album_icon_.pixmap(album_icon_.availableSizes().last()).scaled(CollectionModel::kPrettyCoverSize, CollectionModel::kPrettyCoverSize, Qt::KeepAspectRatio, Qt::SmoothTransformation);
 
 }
 
-void InternetSearchModel::AddResults(const InternetSearch::ResultList &results) {
+void InternetSearchModel::AddResults(const InternetSearchView::ResultList &results) {
 
-  int sort_index = 0;
-
-  for (const InternetSearch::Result &result : results) {
+  for (const InternetSearchView::Result &result : results) {
     QStandardItem *parent = invisibleRootItem();
 
     // Find (or create) the container nodes for this result if we can.
     ContainerKey key;
-    key.provider_index_ = sort_index;
     parent = BuildContainers(result.metadata_, parent, &key);
 
     // Create the item
     QStandardItem *item = new QStandardItem;
     item->setText(result.metadata_.TitleWithCompilationArtist());
     item->setData(QVariant::fromValue(result), Role_Result);
-    item->setData(sort_index, Role_ProviderIndex);
 
     parent->appendRow(item);
 
@@ -76,7 +75,7 @@ void InternetSearchModel::AddResults(const InternetSearch::ResultList &results)
 
 }
 
-QStandardItem *InternetSearchModel::BuildContainers(const Song &s, QStandardItem *parent, ContainerKey *key, int level) {
+QStandardItem *InternetSearchModel::BuildContainers(const Song &s, QStandardItem *parent, ContainerKey *key, const int level) {
 
   if (level >= 3) {
     return parent;
@@ -114,6 +113,22 @@ QStandardItem *InternetSearchModel::BuildContainers(const Song &s, QStandardItem
       has_artist_icon = true;
       break;
 
+    case CollectionModel::GroupBy_Album:
+      display_text = CollectionModel::TextOrUnknown(s.album());
+      sort_text = CollectionModel::SortTextForArtist(s.album());
+      unique_tag = s.album_id();
+      has_album_icon = true;
+      break;
+
+    case CollectionModel::GroupBy_AlbumDisc:{
+      int disc = qMax(0, s.disc());
+      display_text = CollectionModel::PrettyAlbumDisc(s.album(), disc);
+      sort_text = s.album() + CollectionModel::SortTextForNumber(disc);
+      unique_tag = s.album_id();
+      has_album_icon = true;
+      break;
+    }
+
     case CollectionModel::GroupBy_YearAlbum:{
       int year = qMax(0, s.year());
       display_text = CollectionModel::PrettyYearAlbum(year, s.album());
@@ -123,6 +138,16 @@ QStandardItem *InternetSearchModel::BuildContainers(const Song &s, QStandardItem
       break;
     }
 
+    case CollectionModel::GroupBy_YearAlbumDisc:{
+      int year = qMax(0, s.year());
+      int disc = qMax(0, s.disc());
+      display_text = CollectionModel::PrettyYearAlbumDisc(year, s.album(), disc);
+      sort_text = CollectionModel::SortTextForNumber(year) + s.album() + CollectionModel::SortTextForNumber(disc);
+      unique_tag = s.album_id();
+      has_album_icon = true;
+      break;
+    }
+
     case CollectionModel::GroupBy_OriginalYearAlbum:{
       int year = qMax(0, s.effective_originalyear());
       display_text = CollectionModel::PrettyYearAlbum(year, s.album());
@@ -132,20 +157,42 @@ QStandardItem *InternetSearchModel::BuildContainers(const Song &s, QStandardItem
       break;
     }
 
+    case CollectionModel::GroupBy_OriginalYearAlbumDisc:{
+      const int year = qMax(0, s.effective_originalyear());
+      const int disc = qMax(0, s.disc());
+      display_text = CollectionModel::PrettyYearAlbumDisc(year, s.album(), disc);
+      sort_text = CollectionModel::SortTextForNumber(year) + s.album() + CollectionModel::SortTextForNumber(disc);
+      unique_tag = s.album_id();
+      has_album_icon = true;
+      break;
+    }
+
+    case CollectionModel::GroupBy_Disc:
+      display_text = CollectionModel::PrettyDisc(s.disc());
+      sort_text = CollectionModel::SortTextForArtist(display_text);
+      has_album_icon = true;
+      break;
+
     case CollectionModel::GroupBy_Year:{
-      int year = qMax(0, s.year());
+      const int year = qMax(0, s.year());
       display_text = QString::number(year);
       sort_text = CollectionModel::SortTextForNumber(year) + " ";
       break;
     }
 
     case CollectionModel::GroupBy_OriginalYear:{
-      int year = qMax(0, s.effective_originalyear());
+      const int year = qMax(0, s.effective_originalyear());
       display_text = QString::number(year);
       sort_text = CollectionModel::SortTextForNumber(year) + " ";
       break;
     }
 
+    case CollectionModel::GroupBy_Genre:
+      display_text = CollectionModel::TextOrUnknown(s.genre());
+      sort_text = CollectionModel::SortTextForArtist(s.genre());
+      has_album_icon = true;
+      break;
+
     case CollectionModel::GroupBy_Composer:
       display_text = CollectionModel::TextOrUnknown(s.composer());
       sort_text = CollectionModel::SortTextForArtist(s.composer());
@@ -158,51 +205,17 @@ QStandardItem *InternetSearchModel::BuildContainers(const Song &s, QStandardItem
       has_album_icon = true;
       break;
 
-    case CollectionModel::GroupBy_Disc:
-      display_text = s.disc();
-      sort_text = CollectionModel::SortTextForArtist(display_text);
-      has_album_icon = true;
-      break;
-
     case CollectionModel::GroupBy_Grouping:
       display_text = CollectionModel::TextOrUnknown(s.grouping());
       sort_text = CollectionModel::SortTextForArtist(s.grouping());
       has_album_icon = true;
       break;
 
-    case CollectionModel::GroupBy_Genre:
-      display_text = CollectionModel::TextOrUnknown(s.genre());
-      sort_text = CollectionModel::SortTextForArtist(s.genre());
-      has_album_icon = true;
-      break;
-
-    case CollectionModel::GroupBy_Album:
-      display_text = CollectionModel::TextOrUnknown(s.album());
-      sort_text = CollectionModel::SortTextForArtist(s.album());
-      unique_tag = s.album_id();
-      has_album_icon = true;
-      break;
-
     case CollectionModel::GroupBy_FileType:
       display_text = s.TextForFiletype();
       sort_text = display_text;
       break;
 
-    case CollectionModel::GroupBy_Bitrate:
-      display_text = QString(s.bitrate(), 1);
-      sort_text = display_text;
-      break;
-
-    case CollectionModel::GroupBy_Samplerate:
-      display_text = QString(s.samplerate(), 1);
-      sort_text = display_text;
-      break;
-
-    case CollectionModel::GroupBy_Bitdepth:
-      display_text = QString(s.bitdepth(), 1);
-      sort_text = display_text;
-      break;
-
     case CollectionModel::GroupBy_Format:
       if (s.samplerate() <= 0) {
         display_text = s.TextForFiletype();
@@ -218,24 +231,23 @@ QStandardItem *InternetSearchModel::BuildContainers(const Song &s, QStandardItem
       sort_text = display_text;
       break;
 
-    case CollectionModel::GroupBy_AlbumDisc:{
-      int disc = qMax(0, s.disc());
-      display_text = CollectionModel::PrettyAlbumDisc(s.album(), disc);
-      sort_text = s.album() + CollectionModel::SortTextForNumber(disc);
-      unique_tag = s.album_id();
-      has_album_icon = true;
+    case CollectionModel::GroupBy_Samplerate:
+      display_text = QString(s.samplerate(), 1);
+      sort_text = display_text;
       break;
-    }
-    case CollectionModel::GroupBy_YearAlbumDisc:{
-      int year = qMax(0, s.year());
-      int disc = qMax(0, s.disc());
-      display_text = CollectionModel::PrettyYearAlbumDisc(year, s.album(), disc);
-      sort_text = CollectionModel::SortTextForNumber(year) + s.album() + CollectionModel::SortTextForNumber(disc);
-      unique_tag = s.album_id();
-      has_album_icon = true;
+
+    case CollectionModel::GroupBy_Bitdepth:
+      display_text = QString(s.bitdepth(), 1);
+      sort_text = display_text;
       break;
-    }
+
+    case CollectionModel::GroupBy_Bitrate:
+      display_text = QString(s.bitrate(), 1);
+      sort_text = display_text;
+      break;
+
     case CollectionModel::GroupBy_None:
+    case CollectionModel::GroupByCount:
       return parent;
   }
 
@@ -247,7 +259,6 @@ QStandardItem *InternetSearchModel::BuildContainers(const Song &s, QStandardItem
   QStandardItem *container = containers_[*key];
   if (!container) {
     container = new QStandardItem(display_text);
-    container->setData(key->provider_index_, Role_ProviderIndex);
     container->setData(sort_text, CollectionModel::Role_SortText);
     container->setData(group_by_[level], CollectionModel::Role_ContainerType);
 
@@ -273,11 +284,13 @@ QStandardItem *InternetSearchModel::BuildContainers(const Song &s, QStandardItem
 }
 
 void InternetSearchModel::Clear() {
+
   containers_.clear();
   clear();
+
 }
 
-InternetSearch::ResultList InternetSearchModel::GetChildResults(const QModelIndexList &indexes) const {
+InternetSearchView::ResultList InternetSearchModel::GetChildResults(const QModelIndexList &indexes) const {
 
   QList<QStandardItem*> items;
   for (const QModelIndex &index : indexes) {
@@ -287,9 +300,9 @@ InternetSearch::ResultList InternetSearchModel::GetChildResults(const QModelInde
 
 }
 
-InternetSearch::ResultList InternetSearchModel::GetChildResults(const QList<QStandardItem*> &items) const {
+InternetSearchView::ResultList InternetSearchModel::GetChildResults(const QList<QStandardItem*> &items) const {
 
-  InternetSearch::ResultList results;
+  InternetSearchView::ResultList results;
   QSet<const QStandardItem*> visited;
 
   for (QStandardItem *item : items) {
@@ -300,7 +313,7 @@ InternetSearch::ResultList InternetSearchModel::GetChildResults(const QList<QSta
 
 }
 
-void InternetSearchModel::GetChildResults(const QStandardItem *item, InternetSearch::ResultList *results, QSet<const QStandardItem*> *visited) const {
+void InternetSearchModel::GetChildResults(const QStandardItem *item, InternetSearchView::ResultList *results, QSet<const QStandardItem*> *visited) const {
 
   if (visited->contains(item)) {
     return;
@@ -312,7 +325,7 @@ void InternetSearchModel::GetChildResults(const QStandardItem *item, InternetSea
     const QModelIndex parent_proxy_index = proxy_->mapFromSource(item->index());
 
     // Yes - visit all the children, but do so through the proxy so we get them in the right order.
-    for (int i = 0; i < item->rowCount(); ++i) {
+    for (int i = 0 ; i < item->rowCount() ; ++i) {
       const QModelIndex proxy_index = parent_proxy_index.model()->index(i, 0, parent_proxy_index);
       const QModelIndex index = proxy_->mapToSource(proxy_index);
       GetChildResults(itemFromIndex(index), results, visited);
@@ -322,54 +335,41 @@ void InternetSearchModel::GetChildResults(const QStandardItem *item, InternetSea
     // No - maybe it's a song, add its result if valid
     QVariant result = item->data(Role_Result);
     if (result.isValid()) {
-      results->append(result.value<InternetSearch::Result>());
-    }
-    else {
-      // Maybe it's a provider then?
-      bool is_provider;
-      const int sort_index = item->data(Role_ProviderIndex).toInt(&is_provider);
-      if (is_provider) {
-        // Go through all the items (through the proxy to keep them ordered) and add the ones belonging to this provider to our list
-        for (int i = 0; i < proxy_->rowCount(invisibleRootItem()->index()); ++i) {
-          QModelIndex child_index = proxy_->index(i, 0, invisibleRootItem()->index());
-          const QStandardItem *child_item = itemFromIndex(proxy_->mapToSource(child_index));
-          if (child_item->data(Role_ProviderIndex).toInt() == sort_index) {
-            GetChildResults(child_item, results, visited);
-          }
-        }
-      }
+      results->append(result.value<InternetSearchView::Result>());
     }
   }
 
 }
 
 QMimeData *InternetSearchModel::mimeData(const QModelIndexList &indexes) const {
-  return engine_->LoadTracks(GetChildResults(indexes));
+
+  return LoadTracks(GetChildResults(indexes));
+
 }
 
 namespace {
-void GatherResults(const QStandardItem *parent, InternetSearch::ResultList *results) {
+void GatherResults(const QStandardItem *parent, InternetSearchView::ResultList *results) {
 
   QVariant result_variant = parent->data(InternetSearchModel::Role_Result);
   if (result_variant.isValid()) {
-    InternetSearch::Result result = result_variant.value<InternetSearch::Result>();
+    InternetSearchView::Result result = result_variant.value<InternetSearchView::Result>();
     (*results).append(result);
   }
 
-  for (int i = 0; i < parent->rowCount(); ++i) {
+  for (int i = 0 ; i < parent->rowCount() ; ++i) {
     GatherResults(parent->child(i), results);
   }
 }
 }
 
-void InternetSearchModel::SetGroupBy(const CollectionModel::Grouping &grouping, bool regroup_now) {
+void InternetSearchModel::SetGroupBy(const CollectionModel::Grouping &grouping, const bool regroup_now) {
 
   const CollectionModel::Grouping old_group_by = group_by_;
   group_by_ = grouping;
 
   if (regroup_now && group_by_ != old_group_by) {
     // Walk the tree gathering the results we have already
-    InternetSearch::ResultList results;
+    InternetSearchView::ResultList results;
     GatherResults(invisibleRootItem(), &results);
 
     // Reset the model and re-add all the results using the new grouping.
@@ -378,3 +378,27 @@ void InternetSearchModel::SetGroupBy(const CollectionModel::Grouping &grouping,
   }
 
 }
+
+MimeData *InternetSearchModel::LoadTracks(const InternetSearchView::ResultList &results) const {
+
+  if (results.isEmpty()) {
+    return nullptr;
+  }
+
+  SongList songs;
+  QList<QUrl> urls;
+  for (const InternetSearchView::Result &result : results) {
+    songs << result.metadata_;
+    urls << result.metadata_.url();
+  }
+
+  InternetSongMimeData *internet_song_mime_data = new InternetSongMimeData(service_);
+  internet_song_mime_data->songs = songs;
+  MimeData *mime_data = internet_song_mime_data;
+
+  mime_data->setUrls(urls);
+
+  return mime_data;
+
+}
+
