@@ -84,6 +84,10 @@
 #include "songplaylistitem.h"
 #include "tagreadermessages.pb.h"
 
+#include "smartplaylists/playlistgenerator.h"
+#include "smartplaylists/playlistgeneratorinserter.h"
+#include "smartplaylists/playlistgeneratormimedata.h"
+
 #include "internet/internetplaylistitem.h"
 #include "internet/internetsongmimedata.h"
 
@@ -111,7 +115,7 @@ const int Playlist::kUndoItemLimit = 500;
 const qint64 Playlist::kMinScrobblePointNsecs = 31ll * kNsecPerSec;
 const qint64 Playlist::kMaxScrobblePointNsecs = 240ll * kNsecPerSec;
 
-Playlist::Playlist(PlaylistBackend *backend, TaskManager *task_manager, CollectionBackend *collection, int id, const QString &special_type, bool favorite, QObject *parent)
+Playlist::Playlist(PlaylistBackend *backend, TaskManager *task_manager, CollectionBackend *collection, const int id, const QString &special_type, const bool favorite, QObject *parent)
     : QAbstractListModel(parent),
       is_loading_(false),
       proxy_(new PlaylistFilter(this)),
@@ -130,9 +134,12 @@ Playlist::Playlist(PlaylistBackend *backend, TaskManager *task_manager, Collecti
       special_type_(special_type),
       cancel_restore_(false),
       scrobbled_(false),
-      nowplaying_(false),
       scrobble_point_(-1),
-      editing_(-1) {
+      editing_(-1),
+      auto_sort_(false),
+      sort_column_(Column_Title),
+      sort_order_(Qt::AscendingOrder)
+      {
 
   undo_stack_->setUndoLimit(kUndoStackSize);
 
@@ -161,7 +168,7 @@ Playlist::~Playlist() {
 }
 
 template <typename T>
-void Playlist::InsertSongItems(const SongList &songs, int pos, bool play_now, bool enqueue, bool enqueue_next) {
+void Playlist::InsertSongItems(const SongList &songs, const int pos, const bool play_now, const bool enqueue, const bool enqueue_next) {
 
   PlaylistItemList items;
 
@@ -271,6 +278,9 @@ QVariant Playlist::data(const QModelIndex &idx, int role) const {
     case Role_QueuePosition:
       return queue_->PositionOf(idx);
 
+    case Role_CanSetRating:
+      return idx.column() == Column_Rating && items_[idx.row()]->IsLocalCollectionItem() && items_[idx.row()]->Metadata().id() != -1;
+
     case Qt::EditRole:
     case Qt::ToolTipRole:
     case Qt::DisplayRole: {
@@ -314,6 +324,8 @@ QVariant Playlist::data(const QModelIndex &idx, int role) const {
 
         case Column_Source:             return song.source();
 
+        case Column_Rating:             return song.rating();
+
       }
 
       return QVariant();
@@ -331,6 +343,9 @@ QVariant Playlist::data(const QModelIndex &idx, int role) const {
       if (items_[idx.row()]->HasCurrentForegroundColor()) {
         return QBrush(items_[idx.row()]->GetCurrentForegroundColor());
       }
+      if (idx.row() < dynamic_history_length()) {
+        return QBrush(kDynamicHistoryColor);
+      }
 
       return QVariant();
 
@@ -360,38 +375,38 @@ QVariant Playlist::data(const QModelIndex &idx, int role) const {
 }
 
 #ifdef HAVE_MOODBAR
-void Playlist::MoodbarUpdated(const QModelIndex& index) {
-  emit dataChanged(index.sibling(index.row(), Column_Mood), index.sibling(index.row(), Column_Mood));
+void Playlist::MoodbarUpdated(const QModelIndex &idx) {
+  emit dataChanged(idx.sibling(idx.row(), Column_Mood), idx.sibling(idx.row(), Column_Mood));
 }
 #endif
 
-bool Playlist::setData(const QModelIndex &index, const QVariant &value, int role) {
+bool Playlist::setData(const QModelIndex &idx, const QVariant &value, int role) {
 
   Q_UNUSED(role);
 
-  int row = index.row();
+  int row = idx.row();
   PlaylistItemPtr item = item_at(row);
   Song song = item->OriginalMetadata();
 
-  if (index.data() == value) return false;
+  if (idx.data() == value) return false;
 
-  if (!set_column_value(song, static_cast<Column>(index.column()), value)) return false;
+  if (!set_column_value(song, static_cast<Column>(idx.column()), value)) return false;
 
   TagReaderReply *reply = TagReaderClient::Instance()->SaveFile(song.url().toLocalFile(), song);
-  NewClosure(reply, SIGNAL(Finished(bool)), this, SLOT(SongSaveComplete(TagReaderReply*, QPersistentModelIndex)), reply, QPersistentModelIndex(index));
+  NewClosure(reply, SIGNAL(Finished(bool)), this, SLOT(SongSaveComplete(TagReaderReply*, QPersistentModelIndex)), reply, QPersistentModelIndex(idx));
 
   return true;
 
 }
 
-void Playlist::SongSaveComplete(TagReaderReply *reply, const QPersistentModelIndex &index) {
+void Playlist::SongSaveComplete(TagReaderReply *reply, const QPersistentModelIndex &idx) {
 
-  if (reply->is_successful() && index.isValid()) {
+  if (reply->is_successful() && idx.isValid()) {
     if (reply->message().save_file_response().success()) {
-      PlaylistItemPtr item = item_at(index.row());
+      PlaylistItemPtr item = item_at(idx.row());
       if (!item) return;
       QFuture<void> future = item->BackgroundReload();
-      NewClosure(future, this, SLOT(ItemReloadComplete(QPersistentModelIndex)), index);
+      NewClosure(future, this, SLOT(ItemReloadComplete(QPersistentModelIndex)), idx);
     }
     else {
       emit Error(tr("An error occurred writing metadata to '%1'").arg(QString::fromStdString(reply->request_message().save_file_request().filename())));
@@ -401,17 +416,17 @@ void Playlist::SongSaveComplete(TagReaderReply *reply, const QPersistentModelInd
 
 }
 
-void Playlist::ItemReloadComplete(const QPersistentModelIndex &index) {
+void Playlist::ItemReloadComplete(const QPersistentModelIndex &idx) {
 
-  if (index.isValid()) {
+  if (idx.isValid()) {
 
-    PlaylistItemPtr item = item_at(index.row());
+    PlaylistItemPtr item = item_at(idx.row());
     if (item && item->HasTemporaryMetadata()) {  // Update temporary metadata.
       item->UpdateTemporaryMetadata(item->OriginalMetadata());
     }
 
-    emit dataChanged(index, index);
-    emit EditingFinished(index);
+    emit dataChanged(idx, idx);
+    emit EditingFinished(idx);
   }
 
 }
@@ -428,18 +443,18 @@ int Playlist::last_played_row() const {
   return last_played_item_index_.isValid() ? last_played_item_index_.row() : -1;
 }
 
-void Playlist::ShuffleModeChanged(PlaylistSequence::ShuffleMode mode) {
+void Playlist::ShuffleModeChanged(const PlaylistSequence::ShuffleMode mode) {
   is_shuffled_ = (mode != PlaylistSequence::Shuffle_Off);
   ReshuffleIndices();
 }
 
-bool Playlist::FilterContainsVirtualIndex(int i) const {
+bool Playlist::FilterContainsVirtualIndex(const int i) const {
   if (i < 0 || i >= virtual_items_.count()) return false;
 
   return proxy_->filterAcceptsRow(virtual_items_[i], QModelIndex());
 }
 
-int Playlist::NextVirtualIndex(int i, bool ignore_repeat_track) const {
+int Playlist::NextVirtualIndex(int i, const bool ignore_repeat_track) const {
 
   PlaylistSequence::RepeatMode repeat_mode = playlist_sequence_->repeat_mode();
   PlaylistSequence::ShuffleMode shuffle_mode = playlist_sequence_->shuffle_mode();
@@ -483,7 +498,7 @@ int Playlist::NextVirtualIndex(int i, bool ignore_repeat_track) const {
 
 }
 
-int Playlist::PreviousVirtualIndex(int i, bool ignore_repeat_track) const {
+int Playlist::PreviousVirtualIndex(int i, const bool ignore_repeat_track) const {
 
   PlaylistSequence::RepeatMode repeat_mode = playlist_sequence_->repeat_mode();
   PlaylistSequence::ShuffleMode shuffle_mode = playlist_sequence_->shuffle_mode();
@@ -521,7 +536,7 @@ int Playlist::PreviousVirtualIndex(int i, bool ignore_repeat_track) const {
 
 }
 
-int Playlist::next_row(bool ignore_repeat_track) const {
+int Playlist::next_row(const bool ignore_repeat_track) const {
 
   // Any queued items take priority
   if (!queue_->is_empty()) {
@@ -553,7 +568,7 @@ int Playlist::next_row(bool ignore_repeat_track) const {
 
 }
 
-int Playlist::previous_row(bool ignore_repeat_track) const {
+int Playlist::previous_row(const bool ignore_repeat_track) const {
 
   int prev_virtual_index = PreviousVirtualIndex(current_virtual_index_,ignore_repeat_track);
 
@@ -580,28 +595,31 @@ int Playlist::previous_row(bool ignore_repeat_track) const {
 
 }
 
-void Playlist::set_current_row(int i, bool is_stopping) {
+void Playlist::set_current_row(const int i, const AutoScroll autoscroll, const bool is_stopping, const bool force_inform) {
 
   QModelIndex old_current_item_index = current_item_index_;
+  QModelIndex new_current_item_index;
+  if (i != -1) new_current_item_index = QPersistentModelIndex(index(i, 0, QModelIndex()));
 
-  ClearStreamMetadata();
+  if (new_current_item_index != current_item_index_) ClearStreamMetadata();
 
-  if (next_row() != -1 && next_row() != i) {
-    PlaylistItemPtr next_item = item_at(next_row());
+  int nextrow = next_row();
+  if (nextrow != -1 && nextrow != i) {
+    PlaylistItemPtr next_item = item_at(nextrow);
     if (next_item) {
       next_item->ClearTemporaryMetadata();
-      emit dataChanged(index(next_row(), 0), index(next_row(), ColumnCount - 1));
+      emit dataChanged(index(nextrow, 0), index(nextrow, ColumnCount - 1));
     }
   }
 
-  current_item_index_ = QPersistentModelIndex(index(i, 0, QModelIndex()));
+  current_item_index_ = new_current_item_index;
 
   // if the given item is the first in the queue, remove it from the queue
-  if (current_item_index_.row() == queue_->PeekNext()) {
+  if (current_item_index_.isValid() && current_item_index_.row() == queue_->PeekNext()) {
     queue_->TakeNext();
   }
 
-  if (current_item_index_ == old_current_item_index) {
+  if (current_item_index_ == old_current_item_index && !force_inform) {
     UpdateScrobblePoint();
     return;
   }
@@ -631,7 +649,34 @@ void Playlist::set_current_row(int i, bool is_stopping) {
   }
 
   if (current_item_index_.isValid() && !is_stopping) {
-    InformOfCurrentSongChange();
+    InformOfCurrentSongChange(autoscroll, false);
+  }
+
+  // The structure of a dynamic playlist is as follows:
+  //   history - active song - future
+  // We have to ensure that this invariant is maintained.
+  if (dynamic_playlist_ && current_item_index_.isValid()) {
+
+    // When advancing to the next track
+    if (old_current_item_index.isValid() && i > old_current_item_index.row()) {
+      // Move the new item one position ahead of the last item in the history.
+      MoveItemWithoutUndo(current_item_index_.row(), dynamic_history_length());
+
+      // Compute the number of new items that have to be inserted
+      // This is not necessarily 1 because the user might have added or removed items manually.
+      // Note that the future excludes the current item.
+      const int count = dynamic_history_length() + 1 + dynamic_playlist_->GetDynamicFuture() - items_.count();
+      if (count > 0) {
+        InsertDynamicItems(count);
+      }
+
+      // Shrink the history, again this is not necessarily by 1, because the user might have moved items by hand.
+      const int remove_count = dynamic_history_length() - dynamic_playlist_->GetDynamicHistory();
+      if (0 < remove_count) RemoveItemsWithoutUndo(0, remove_count);
+    }
+
+    // the above actions make all commands on the undo stack invalid, so we better clear it.
+    undo_stack_->clear();
   }
 
   if (current_item_index_.isValid()) {
@@ -643,15 +688,26 @@ void Playlist::set_current_row(int i, bool is_stopping) {
 
 }
 
-Qt::ItemFlags Playlist::flags(const QModelIndex &index) const {
+void Playlist::InsertDynamicItems(const int count) {
 
-  Qt::ItemFlags flags = Qt::ItemIsEnabled | Qt::ItemIsSelectable;
+  PlaylistGeneratorInserter* inserter = new PlaylistGeneratorInserter(task_manager_, collection_, this);
+  connect(inserter, SIGNAL(Error(QString)), SIGNAL(Error(QString)));
+  connect(inserter, SIGNAL(PlayRequested(QModelIndex)), SIGNAL(PlayRequested(QModelIndex)));
 
-  if (column_is_editable(static_cast<Column>(index.column()))) flags |= Qt::ItemIsEditable;
+  inserter->Load(this, -1, false, false, false, dynamic_playlist_, count);
 
-  if (index.isValid()) return flags | Qt::ItemIsDragEnabled;
+}
 
-  return Qt::ItemIsDropEnabled;
+Qt::ItemFlags Playlist::flags(const QModelIndex &idx) const {
+
+  if (idx.isValid()) {
+    Qt::ItemFlags flags = Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled;
+    if (item_at(idx.row())->Metadata().IsEditable() && column_is_editable(static_cast<Column>(idx.column()))) flags |= Qt::ItemIsEditable;
+    return flags;
+  }
+  else {
+    return Qt::ItemIsDropEnabled;
+  }
 
 }
 
@@ -696,6 +752,9 @@ bool Playlist::dropMimeData(const QMimeData *data, Qt::DropAction action, int ro
   else if (const InternetSongMimeData* internet_song_data = qobject_cast<const InternetSongMimeData*>(data)) {
     InsertInternetItems(internet_song_data->service, internet_song_data->songs, row, play_now, enqueue_now, enqueue_next_now);
   }
+  else if (const PlaylistGeneratorMimeData *generator_data = qobject_cast<const PlaylistGeneratorMimeData*>(data)) {
+    InsertSmartPlaylist(generator_data->generator_, row, play_now, enqueue_now, enqueue_next_now);
+  }
   else if (data->hasFormat(kRowsMimetype)) {
     // Dragged from the playlist
     // Rearranging it is tricky...
@@ -758,7 +817,7 @@ bool Playlist::dropMimeData(const QMimeData *data, Qt::DropAction action, int ro
 
 }
 
-void Playlist::InsertUrls(const QList<QUrl> &urls, int pos, bool play_now, bool enqueue, bool enqueue_next) {
+void Playlist::InsertUrls(const QList<QUrl> &urls, const int pos, const bool play_now, const bool enqueue, const bool enqueue_next) {
 
   SongLoaderInserter *inserter = new SongLoaderInserter(task_manager_, collection_, backend_->app()->player());
   connect(inserter, SIGNAL(Error(QString)), SIGNAL(Error(QString)));
@@ -767,7 +826,35 @@ void Playlist::InsertUrls(const QList<QUrl> &urls, int pos, bool play_now, bool
 
 }
 
-void Playlist::MoveItemWithoutUndo(int source, int dest) {
+void Playlist::InsertSmartPlaylist(PlaylistGeneratorPtr generator, const int pos, const bool play_now, const bool enqueue, const bool enqueue_next) {
+
+  // Hack: If the generator hasn't got a collection set then use the main one
+  if (!generator->collection()) {
+    generator->set_collection(collection_);
+  }
+
+  PlaylistGeneratorInserter *inserter = new PlaylistGeneratorInserter(task_manager_, collection_, this);
+  connect(inserter, SIGNAL(Error(QString)), SIGNAL(Error(QString)));
+
+  inserter->Load(this, pos, play_now, enqueue, enqueue_next, generator);
+
+  if (generator->is_dynamic()) {
+    TurnOnDynamicPlaylist(generator);
+  }
+
+}
+
+void Playlist::TurnOnDynamicPlaylist(PlaylistGeneratorPtr gen) {
+
+  dynamic_playlist_ = gen;
+  playlist_sequence_->SetUsingDynamicPlaylist(true);
+  ShuffleModeChanged(PlaylistSequence::Shuffle_Off);
+  emit DynamicModeChanged(true);
+  Save();
+
+}
+
+void Playlist::MoveItemWithoutUndo(const int source, const int dest) {
   MoveItemsWithoutUndo(QList<int>() << source, dest);
 }
 
@@ -871,7 +958,7 @@ void Playlist::MoveItemsWithoutUndo(int start, const QList<int> &dest_rows) {
 
 }
 
-void Playlist::InsertItems(const PlaylistItemList &itemsIn, int pos, bool play_now, bool enqueue, bool enqueue_next) {
+void Playlist::InsertItems(const PlaylistItemList &itemsIn, const int pos, const bool play_now, const bool enqueue, const bool enqueue_next) {
 
   if (itemsIn.isEmpty())
     return;
@@ -927,11 +1014,11 @@ void Playlist::InsertItems(const PlaylistItemList &itemsIn, int pos, bool play_n
     undo_stack_->push(new PlaylistUndoCommands::InsertItems(this, items, pos, enqueue, enqueue_next));
   }
 
-  if (play_now) emit PlayRequested(index(start, 0));
+  if (play_now) emit PlayRequested(index(start, 0), AutoScroll_Maybe);
 
 }
 
-void Playlist::InsertItemsWithoutUndo(const PlaylistItemList &items, int pos, bool enqueue, bool enqueue_next) {
+void Playlist::InsertItemsWithoutUndo(const PlaylistItemList &items, const int pos, const bool enqueue, const bool enqueue_next) {
 
   if (items.isEmpty()) return;
 
@@ -976,7 +1063,13 @@ void Playlist::InsertItemsWithoutUndo(const PlaylistItemList &items, int pos, bo
   }
 
   Save();
-  ReshuffleIndices();
+
+  if (auto_sort_) {
+    sort(sort_column_, sort_order_);
+  }
+  else {
+    ReshuffleIndices();
+  }
 
 }
 
@@ -1071,11 +1164,11 @@ QMimeData *Playlist::mimeData(const QModelIndexList &indexes) const {
 
   QList<QUrl> urls;
   QList<int> rows;
-  for (const QModelIndex &index : indexes) {
-    if (index.column() != first_column) continue;
+  for (const QModelIndex &idx : indexes) {
+    if (idx.column() != first_column) continue;
 
-    urls << items_[index.row()]->Url();
-    rows << index.row();
+    urls << items_[idx.row()]->Url();
+    rows << idx.row();
   }
 
   QBuffer buf;
@@ -1097,7 +1190,7 @@ QMimeData *Playlist::mimeData(const QModelIndexList &indexes) const {
 
 }
 
-bool Playlist::CompareItems(int column, Qt::SortOrder order, std::shared_ptr<PlaylistItem> _a, std::shared_ptr<PlaylistItem> _b) {
+bool Playlist::CompareItems(const int column, const Qt::SortOrder order, std::shared_ptr<PlaylistItem> _a, std::shared_ptr<PlaylistItem> _b) {
 
   std::shared_ptr<PlaylistItem> a = order == Qt::AscendingOrder ? _a : _b;
   std::shared_ptr<PlaylistItem> b = order == Qt::AscendingOrder ? _b : _a;
@@ -1138,6 +1231,9 @@ bool Playlist::CompareItems(int column, Qt::SortOrder order, std::shared_ptr<Pla
 
     case Column_Comment:      strcmp(comment);
     case Column_Source:       cmp(source);
+
+    case Column_Rating:       cmp(rating);
+
     default: qLog(Error) << "No such column" << column;
   }
 
@@ -1148,7 +1244,7 @@ bool Playlist::CompareItems(int column, Qt::SortOrder order, std::shared_ptr<Pla
 
 }
 
-bool Playlist::ComparePathDepths(Qt::SortOrder order, std::shared_ptr<PlaylistItem> _a, std::shared_ptr<PlaylistItem> _b) {
+bool Playlist::ComparePathDepths(const Qt::SortOrder order, std::shared_ptr<PlaylistItem> _a, std::shared_ptr<PlaylistItem> _b) {
 
   std::shared_ptr<PlaylistItem> a = order == Qt::AscendingOrder ? _a : _b;
   std::shared_ptr<PlaylistItem> b = order == Qt::AscendingOrder ? _b : _a;
@@ -1195,13 +1291,14 @@ QString Playlist::column_name(Column column) {
     case Column_Comment:      return tr("Comment");
     case Column_Source:       return tr("Source");
     case Column_Mood:         return tr("Mood");
+    case Column_Rating:       return tr("Rating");
     default:                  qLog(Error) << "No such column" << column;;
   }
   return "";
 
 }
 
-QString Playlist::abbreviated_column_name(Column column) {
+QString Playlist::abbreviated_column_name(const Column column) {
 
   const QString &column_name = Playlist::column_name(column);
 
@@ -1220,11 +1317,17 @@ QString Playlist::abbreviated_column_name(Column column) {
 
 void Playlist::sort(int column, Qt::SortOrder order) {
 
+  sort_column_ = column;
+  sort_order_ = order;
+
   if (ignore_sorting_) return;
 
   PlaylistItemList new_items(items_);
   PlaylistItemList::iterator begin = new_items.begin();
 
+  if (dynamic_playlist_ && current_item_index_.isValid())
+    begin += current_item_index_.row() + 1;
+
   if (column == Column_Album) {
     // When sorting by album, also take into account discs and tracks.
     std::stable_sort(begin, new_items.end(), std::bind(&Playlist::CompareItems, Column_Track, order, _1, _2));
@@ -1276,7 +1379,7 @@ void Playlist::Paused() { SetCurrentIsPaused(true); }
 
 void Playlist::Stopped() { SetCurrentIsPaused(false); }
 
-void Playlist::SetCurrentIsPaused(bool paused) {
+void Playlist::SetCurrentIsPaused(const bool paused) {
 
   if (paused == current_is_paused_) return;
 
@@ -1290,7 +1393,7 @@ void Playlist::Save() const {
 
   if (!backend_ || is_loading_) return;
 
-  backend_->SavePlaylistAsync(id_, items_, last_played_row());
+  backend_->SavePlaylistAsync(id_, items_, last_played_row(), dynamic_playlist_);
 
 }
 
@@ -1333,6 +1436,22 @@ void Playlist::ItemsLoaded(QFuture<PlaylistItemList> future) {
   // The newly loaded list of items might be shorter than it was before so look out for a bad last_played index
   last_played_item_index_ = p.last_played == -1 || p.last_played >= rowCount() ? QModelIndex() : index(p.last_played);
 
+  if (p.dynamic_type == PlaylistGenerator::Type_Query) {
+    PlaylistGeneratorPtr gen = PlaylistGenerator::Create(p.dynamic_type);
+    if (gen) {
+
+      CollectionBackend *backend = nullptr;
+      if (p.dynamic_backend == collection_->songs_table()) backend = collection_;
+
+      if (backend) {
+        gen->set_collection(backend);
+        gen->Load(p.dynamic_data);
+        TurnOnDynamicPlaylist(gen);
+      }
+
+    }
+  }
+
   emit RestoreFinished();
 
   QSettings s;
@@ -1469,7 +1588,7 @@ PlaylistItemList Playlist::RemoveItemsWithoutUndo(const int row, const int count
 
 }
 
-void Playlist::StopAfter(int row) {
+void Playlist::StopAfter(const int row) {
 
   QModelIndex old_stop_after = stop_after_;
 
@@ -1489,28 +1608,10 @@ void Playlist::SetStreamMetadata(const QUrl &url, const Song &song, const bool m
 
   if (!current_item() || current_item()->Url() != url) return;
 
-  //qLog(Debug) << "Setting temporary metadata for" << url;
-
   bool update_scrobble_point = song.length_nanosec() != current_item_metadata().length_nanosec();
-
   current_item()->SetTemporaryMetadata(song);
-
-  if (minor) {
-    if (editing_ != current_item_index_.row()) {
-      emit dataChanged(index(current_item_index_.row(), 0), index(current_item_index_.row(), ColumnCount - 1));
-    }
-    // if the song is invalid, we won't play it - there's no point in informing anybody about the change
-    const Song metadata(current_item_metadata());
-    if (metadata.is_valid()) {
-      emit SongMetadataChanged(metadata);
-    }
-  }
-  else {
-    update_scrobble_point = true;
-    InformOfCurrentSongChange();
-  }
-
   if (update_scrobble_point) UpdateScrobblePoint();
+  InformOfCurrentSongChange(AutoScroll_Never, minor);
 
 }
 
@@ -1571,10 +1672,29 @@ void Playlist::Clear() {
     undo_stack_->push(new PlaylistUndoCommands::RemoveItems(this, 0, count));
   }
 
+  TurnOffDynamicPlaylist();
+
   Save();
 
 }
 
+void Playlist::RepopulateDynamicPlaylist() {
+
+  if (!dynamic_playlist_) return;
+
+  RemoveItemsNotInQueue();
+  InsertSmartPlaylist(dynamic_playlist_);
+
+}
+
+void Playlist::ExpandDynamicPlaylist() {
+
+  if (!dynamic_playlist_) return;
+
+  InsertDynamicItems(5);
+
+}
+
 void Playlist::RemoveItemsNotInQueue() {
 
   if (queue_->is_empty() && !current_item_index_.isValid()) {
@@ -1610,10 +1730,16 @@ void Playlist::ReloadItems(const QList<int> &rows) {
   for (int row : rows) {
     PlaylistItemPtr item = item_at(row);
 
+    Song old_metadata = item->Metadata();
+
     item->Reload();
 
     if (row == current_row()) {
-      InformOfCurrentSongChange();
+      const bool minor = old_metadata.title() == item->Metadata().title() &&
+                         old_metadata.albumartist() == item->Metadata().albumartist() &&
+                         old_metadata.artist() == item->Metadata().artist() &&
+                         old_metadata.album() == item->Metadata().album();
+      InformOfCurrentSongChange(AutoScroll_Never, minor);
     }
     else {
       emit dataChanged(index(row, 0), index(row, ColumnCount - 1));
@@ -1649,6 +1775,9 @@ void Playlist::Shuffle() {
     begin = 1;
   }
 
+  if (dynamic_playlist_ && current_item_index_.isValid())
+    begin += current_item_index_.row() + 1;
+
   const int count = items_.count();
   for (int i = begin; i < count; ++i) {
     int new_pos = i + (rand() % (count - i));
@@ -1661,7 +1790,7 @@ void Playlist::Shuffle() {
 }
 
 namespace {
-bool AlbumShuffleComparator(const QMap<QString, int> &album_key_positions, const QMap<int, QString> &album_keys, int left, int right) {
+bool AlbumShuffleComparator(const QMap<QString, int> &album_key_positions, const QMap<int, QString> &album_keys, const int left, const int right) {
 
   const int left_pos = album_key_positions[album_keys[left]];
   const int right_pos = album_key_positions[album_keys[right]];
@@ -1777,7 +1906,7 @@ quint64 Playlist::GetTotalLength() const {
 
 }
 
-PlaylistItemList Playlist::collection_items_by_id(int id) const {
+PlaylistItemList Playlist::collection_items_by_id(const int id) const {
   return collection_items_by_id_.values(id);
 }
 
@@ -1791,15 +1920,15 @@ void Playlist::TracksAboutToBeDequeued(const QModelIndex&, int begin, int end) {
 
 void Playlist::TracksDequeued() {
 
-  for (const QModelIndex &index : temp_dequeue_change_indexes_) {
-    emit dataChanged(index, index);
+  for (const QModelIndex &idx : temp_dequeue_change_indexes_) {
+    emit dataChanged(idx, idx);
   }
   temp_dequeue_change_indexes_.clear();
   emit QueueChanged();
 
 }
 
-void Playlist::TracksEnqueued(const QModelIndex&, int begin, int end) {
+void Playlist::TracksEnqueued(const QModelIndex&, const int begin, const int end) {
 
   const QModelIndex &b = queue_->mapToSource(queue_->index(begin, Column_Title));
   const QModelIndex &e = queue_->mapToSource(queue_->index(end, Column_Title));
@@ -1810,8 +1939,17 @@ void Playlist::TracksEnqueued(const QModelIndex&, int begin, int end) {
 void Playlist::QueueLayoutChanged() {
 
   for (int i = 0; i < queue_->rowCount(); ++i) {
-    const QModelIndex &index = queue_->mapToSource(queue_->index(i, Column_Title));
-    emit dataChanged(index, index);
+    const QModelIndex &idx = queue_->mapToSource(queue_->index(i, Column_Title));
+    emit dataChanged(idx, idx);
+  }
+
+}
+
+void Playlist::ItemChanged(const int row) {
+
+  QModelIndex idx = index(row, ColumnCount - 1);
+  if (idx.isValid()) {
+    emit dataChanged(index(row, 0), index(row, ColumnCount - 1));
   }
 
 }
@@ -1820,23 +1958,28 @@ void Playlist::ItemChanged(PlaylistItemPtr item) {
 
   for (int row = 0; row < items_.count(); ++row) {
     if (items_[row] == item) {
-      QModelIndex idx = index(row, ColumnCount - 1);
-      if (idx.isValid()) {
-        emit dataChanged(index(row, 0), index(row, ColumnCount - 1));
-      }
+      ItemChanged(row);
     }
   }
 
 }
 
-void Playlist::InformOfCurrentSongChange() {
-
-  emit dataChanged(index(current_item_index_.row(), 0), index(current_item_index_.row(), ColumnCount - 1));
+void Playlist::InformOfCurrentSongChange(const AutoScroll autoscroll, const bool minor) {
 
   // if the song is invalid, we won't play it - there's no point in informing anybody about the change
   const Song metadata(current_item_metadata());
   if (metadata.is_valid()) {
-    emit CurrentSongChanged(metadata);
+    if (minor) {
+      emit SongMetadataChanged(metadata);
+      if (editing_ != current_item_index_.row()) {
+        emit dataChanged(index(current_item_index_.row(), 0), index(current_item_index_.row(), ColumnCount - 1));
+      }
+    }
+    else {
+      emit CurrentSongChanged(metadata);
+      emit MaybeAutoscroll(autoscroll);
+      emit dataChanged(index(current_item_index_.row(), 0), index(current_item_index_.row(), ColumnCount - 1));
+    }
   }
 
 }
@@ -1954,7 +2097,7 @@ void Playlist::RemoveUnavailableSongs() {
 
 }
 
-bool Playlist::ApplyValidityOnCurrentSong(const QUrl &url, bool valid) {
+bool Playlist::ApplyValidityOnCurrentSong(const QUrl &url, const bool valid) {
 
   PlaylistItemPtr current = current_item();
 
@@ -2016,7 +2159,6 @@ void Playlist::UpdateScrobblePoint(const qint64 seek_point_nanosec) {
     }
   }
 
-  nowplaying_ = false;
   scrobbled_ = false;
 
 }
@@ -2034,3 +2176,48 @@ void Playlist::AlbumCoverLoaded(const Song &song, const AlbumCoverLoaderResult &
   }
 
 }
+
+int Playlist::dynamic_history_length() const {
+  return dynamic_playlist_ && last_played_item_index_.isValid() ? last_played_item_index_.row() + 1 : 0;
+}
+
+void Playlist::TurnOffDynamicPlaylist() {
+
+  dynamic_playlist_.reset();
+
+  if (playlist_sequence_) {
+    playlist_sequence_->SetUsingDynamicPlaylist(false);
+    ShuffleModeChanged(playlist_sequence_->shuffle_mode());
+  }
+
+  emit DynamicModeChanged(false);
+  Save();
+
+}
+
+void Playlist::RateSong(const QModelIndex &idx, const double rating) {
+
+  if (has_item_at(idx.row())) {
+    PlaylistItemPtr item = item_at(idx.row());
+    if (item && item->IsLocalCollectionItem() && item->Metadata().id() != -1) {
+      collection_->UpdateSongRatingAsync(item->Metadata().id(), rating);
+    }
+  }
+
+}
+
+void Playlist::RateSongs(const QModelIndexList &index_list, const double rating) {
+
+  QList<int> id_list;
+  for (const QModelIndex &idx : index_list) {
+    const int row = idx.row();
+    if (has_item_at(row)) {
+      PlaylistItemPtr item = item_at(row);
+      if (item && item->IsLocalCollectionItem() && item->Metadata().id() != -1) {
+        id_list << item->Metadata().id();
+      }
+    }
+  }
+  collection_->UpdateSongsRatingAsync(id_list, rating);
+
+}
