@@ -72,6 +72,8 @@ class QTimerEvent;
 class Application;
 class CollectionBackend;
 class PlaylistHeader;
+class DynamicPlaylistControls;
+class RatingItemDelegate;
 
 // This proxy style works around a bug/feature introduced in Qt 4.7's QGtkStyle
 // that uses Gtk to paint row backgrounds, ignoring any custom brush or palette the caller set in the QStyleOption.
@@ -98,7 +100,7 @@ class PlaylistView : public QTreeView {
 
   static ColumnAlignmentMap DefaultColumnAlignment();
 
-  void SetApplication(Application *app);
+  void Init(Application *app);
   void SetItemDelegates();
   void SetPlaylist(Playlist *playlist);
   void RemoveSelected();
@@ -109,28 +111,27 @@ class PlaylistView : public QTreeView {
   AppearanceSettingsPage::BackgroundImageType background_image_type() const { return background_image_type_; }
   Qt::Alignment column_alignment(int section) const;
 
-  void ResetColumns();
+  void ResetHeaderState();
 
   // QTreeView
   void setModel(QAbstractItemModel *model) override;
 
  public slots:
   void ReloadSettings();
-  void SaveGeometry();
   void SaveSettings();
-  void SetColumnAlignment(int section, Qt::Alignment alignment);
+  void SetColumnAlignment(const int section, const Qt::Alignment alignment);
   void JumpToCurrentlyPlayingTrack();
-  void edit(const QModelIndex &index) { return QAbstractItemView::edit(index); }
+  void edit(const QModelIndex &idx) { return QAbstractItemView::edit(idx); }
 
  signals:
-  void PlayItem(const QModelIndex &index);
-  void PlayPause();
-  void RightClicked(const QPoint &global_pos, const QModelIndex &index);
+  void PlayItem(QModelIndex idx, Playlist::AutoScroll autoscroll);
+  void PlayPause(Playlist::AutoScroll autoscroll = Playlist::AutoScroll_Never);
+  void RightClicked(QPoint global_pos, QModelIndex idx);
   void SeekForward();
   void SeekBackward();
   void FocusOnFilterSignal(QKeyEvent *event);
   void BackgroundPropertyChanged();
-  void ColumnAlignmentChanged(const ColumnAlignmentMap &alignment);
+  void ColumnAlignmentChanged(ColumnAlignmentMap alignment);
 
  protected:
   // QWidget
@@ -149,26 +150,28 @@ class PlaylistView : public QTreeView {
   void dropEvent(QDropEvent *event) override;
   bool eventFilter(QObject *object, QEvent *event) override;
   void focusInEvent(QFocusEvent *event) override;
+  void resizeEvent(QResizeEvent *event) override;
 
   // QTreeView
   void drawTree(QPainter *painter, const QRegion &region) const;
-  void drawRow(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &index) const override;
+  void drawRow(QPainter *painter, const QStyleOptionViewItem &option, const QModelIndex &idx) const override;
 
   // QAbstractScrollArea
   void scrollContentsBy(int dx, int dy) override;
 
   // QAbstractItemView
   void rowsInserted(const QModelIndex &parent, int start, int end) override;
-  bool edit(const QModelIndex &index, QAbstractItemView::EditTrigger trigger, QEvent *event) override;
+  bool edit(const QModelIndex &idx, QAbstractItemView::EditTrigger trigger, QEvent *event) override;
   void closeEditor(QWidget *editor, QAbstractItemDelegate::EndEditHint hint) override;
 
  private slots:
+  void SetHeaderState();
   void InhibitAutoscrollTimeout();
-  void MaybeAutoscroll();
+  void MaybeAutoscroll(const Playlist::AutoScroll autoscroll);
   void InvalidateCachedCurrentPixmap();
   void PlaylistDestroyed();
-  void StretchChanged(bool stretch);
-  void FadePreviousBackgroundImage(qreal value);
+  void StretchChanged(const bool stretch);
+  void FadePreviousBackgroundImage(const qreal value);
   void StopGlowing();
   void StartGlowing();
   void JumpToLastPlayedTrack();
@@ -177,12 +180,18 @@ class PlaylistView : public QTreeView {
   void Stopped();
   void SongChanged(const Song &song);
   void AlbumCoverLoaded(const Song &song, AlbumCoverLoaderResult result = AlbumCoverLoaderResult());
+  void DynamicModeChanged(const bool dynamic);
+  void SetRatingLockStatus(const bool state);
+  void RatingHoverIn(const QModelIndex &idx, const QPoint &pos);
+  void RatingHoverOut();
 
  private:
-  void LoadGeometry();
+  void LoadHeaderState();
+  void RestoreHeaderState();
+
   void ReloadBarPixmaps();
   QList<QPixmap> LoadBarPixmap(const QString &filename);
-  void UpdateCachedCurrentRowPixmap(QStyleOptionViewItem option, const QModelIndex &index);
+  void UpdateCachedCurrentRowPixmap(QStyleOptionViewItem option, const QModelIndex &idx);
 
   void set_background_image_type(AppearanceSettingsPage::BackgroundImageType bg) {
     background_image_type_ = bg;
@@ -204,6 +213,8 @@ class PlaylistView : public QTreeView {
   QModelIndex NextEditableIndex(const QModelIndex &current);
   QModelIndex PrevEditableIndex(const QModelIndex &current);
 
+  void RepositionDynamicControls();
+
   Application *app_;
   PlaylistProxyStyle *style_;
   Playlist *playlist_;
@@ -219,11 +230,12 @@ class PlaylistView : public QTreeView {
   int blur_radius_;
   int opacity_level_;
 
-  bool initialized_;
   bool background_initialized_;
-  bool setting_initial_header_layout_;
+  bool set_initial_header_layout_;
   bool read_only_settings_;
-  bool state_loaded_;
+  bool header_state_loaded_;
+  bool header_state_restored_;
+  bool header_state_readonly_;
 
   QImage background_image_;
   QImage current_song_cover_art_;
@@ -245,6 +257,7 @@ class PlaylistView : public QTreeView {
 
   bool glow_enabled_;
   bool select_track_;
+  bool auto_sort_;
 
   bool currently_glowing_;
   QBasicTimer glow_timer_;
@@ -272,12 +285,16 @@ class PlaylistView : public QTreeView {
   int drop_indicator_row_;
   bool drag_over_;
 
+  int header_state_version_;
+  QByteArray header_state_;
   ColumnAlignmentMap column_alignment_;
-
-  QByteArray state_;
+  bool rating_locked_;
 
   Song song_playing_;
 
+  DynamicPlaylistControls *dynamic_controls_;
+  RatingItemDelegate *rating_delegate_;
+
 };
 
 #endif  // PLAYLISTVIEW_H
