@@ -43,13 +43,13 @@ class XSPFParser : public XMLParser {
  public:
   explicit XSPFParser(CollectionBackendInterface *collection, QObject *parent = nullptr);
 
-  QString name() const { return "XSPF"; }
-  QStringList file_extensions() const { return QStringList() << "xspf"; }
+  QString name() const override { return "XSPF"; }
+  QStringList file_extensions() const override { return QStringList() << "xspf"; }
 
-  bool TryMagic(const QByteArray &data) const;
+  bool TryMagic(const QByteArray &data) const override;
 
-  SongList Load(QIODevice *device, const QString &playlist_path = "", const QDir &dir = QDir()) const;
-  void Save(const SongList &songs, QIODevice *device, const QDir &dir = QDir(), Playlist::Path path_type = Playlist::Path_Automatic) const;
+  SongList Load(QIODevice *device, const QString &playlist_path = "", const QDir &dir = QDir()) const override;
+  void Save(const SongList &songs, QIODevice *device, const QDir &dir = QDir(), Playlist::Path path_type = Playlist::Path_Automatic) const override;
 
  private:
   Song ParseTrack(QXmlStreamReader *reader, const QDir &dir) const;
