@@ -279,7 +279,7 @@ void ScrobblingAPI20::AuthenticateReplyFinished(QNetworkReply *reply) {
       QString error;
       QJsonParseError json_error;
       QJsonDocument json_doc = QJsonDocument::fromJson(data, &json_error);
-      if (json_error.error == QJsonParseError::NoError && !json_doc.isNull() && !json_doc.isEmpty() && json_doc.isObject()) {
+      if (json_error.error == QJsonParseError::NoError && !json_doc.isEmpty() && json_doc.isObject()) {
         QJsonObject json_obj = json_doc.object();
         if (json_obj.contains("error") && json_obj.contains("message")) {
           int code = json_obj["error"].toInt();
@@ -416,7 +416,7 @@ QByteArray ScrobblingAPI20::GetReplyData(QNetworkReply *reply) {
       QJsonParseError json_error;
       QJsonDocument json_doc = QJsonDocument::fromJson(data, &json_error);
       int error_code = -1;
-      if (json_error.error == QJsonParseError::NoError && !json_doc.isNull() && !json_doc.isEmpty() && json_doc.isObject()) {
+      if (json_error.error == QJsonParseError::NoError && !json_doc.isEmpty() && json_doc.isObject()) {
         QJsonObject json_obj = json_doc.object();
         if (json_obj.contains("error") && json_obj.contains("message")) {
           error_code = json_obj["error"].toInt();
@@ -1013,7 +1013,7 @@ void ScrobblingAPI20::Error(const QString &error, const QVariant &debug) {
 
 }
 
-QString ScrobblingAPI20::ErrorString(const ScrobbleErrorCode error) const {
+QString ScrobblingAPI20::ErrorString(const ScrobbleErrorCode error) {
 
   switch (error) {
     case ScrobbleErrorCode::NoError:
