@@ -35,6 +35,7 @@
 #include <QUrl>
 #include <QNetworkAccessManager>
 #include <QSslError>
+#include <QDateTime>
 
 #include "core/song.h"
 #include "internet/internetservice.h"
@@ -45,6 +46,7 @@ class QNetworkReply;
 class Application;
 class SubsonicUrlHandler;
 class SubsonicRequest;
+class SubsonicScrobbleRequest;
 class CollectionBackend;
 class CollectionModel;
 
@@ -79,6 +81,7 @@ class SubsonicService : public InternetService {
   QSortFilterProxyModel *songs_collection_sort_model() override { return collection_sort_model_; }
 
   void CheckConfiguration();
+  void Scrobble(const QString &song_id, const bool submission, const QDateTime time);
 
  public slots:
   void ShowConfig() override;
@@ -116,6 +119,7 @@ class SubsonicService : public InternetService {
   QSortFilterProxyModel *collection_sort_model_;
 
   std::shared_ptr<SubsonicRequest> songs_request_;
+  std::shared_ptr<SubsonicScrobbleRequest> scrobble_request_;
 
   QUrl server_url_;
   QString username_;
